package gkdtree;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

/**
 *
 * Inner nodes of the kd-tree.
 * 
 * @author DarkMatt
 *
 */

public class GKDInnerNode extends GKDNode {

	public float min, max;
	
	public GKDInnerNode(int dimension, float min, float max, float cut) {
		this.dimension = dimension;
		this.min = min;
		this.max = max;
		this.cut = cut;
		//if (dimension != 3) this.cut -= 1.0f; //TODO: very unintuitive! and only for 2D
		leftChild = -1;
		rightChild = -1;
	}
	
	public void setLeftNode(int index) {
		leftChild = index;
	}
	
	public void setRightNode(int index) {
		rightChild = index;
	}
	
	public void applyScale(float[] scale) {
		this.scale = scale.clone();
		min *= scale[dimension];
		max *= scale[dimension];
		cut *= scale[dimension];
	}
	
	public void undoScale() {
		min /= scale[dimension];
		max /= scale[dimension];
		cut /= scale[dimension];
		Arrays.fill(scale, 1.0f);
	}
	
	public void query(float[] pos, float sigma, int num_samples, 
			ArrayList<Result> results, float p, ArrayList<GKDNode> nodes) {
		p = 1; //probably wrong?
		
		float cdfMin = NormalDist.normcdf((min - pos[dimension])/sigma);
		float cdfMax = NormalDist.normcdf((max - pos[dimension])/sigma);
		float cdfCut = NormalDist.normcdf((cut - pos[dimension])/sigma);
		
		float pLeft = (cdfCut - cdfMin)/(cdfMax - cdfMin);
		/*if (pLeft > 1) {
			System.out.println("this cant happen");
		}*/
		float expectedLeft = pLeft*(float)num_samples;
		
		int samplesLeft = (int)expectedLeft;
		int samplesRight = (int)((float)num_samples - expectedLeft);
		
		if (samplesLeft + samplesRight < num_samples) {
			if ((float)Math.random() < expectedLeft - (float)samplesLeft) {
				samplesLeft++;
			} else {
				samplesRight++;
			}
		} 
		
		//can't happen.. in children no recursion
		if (samplesLeft > 0)// && leftChild != -1)
			nodes.get(leftChild).query(pos, sigma, samplesLeft, results, p*pLeft, nodes);
		if (samplesRight > 0)// && rightChild != -1)
			nodes.get(rightChild).query(pos, sigma, samplesRight, results, p*(1-pLeft), nodes);
	}
	
}
