package gkdtree;

/**
 *
 * Abstraction for n-dimensional data. 
 * To handle offsets and indexing via linear or n-dimensional indices.
 * 
 * @author DarkMatt
 *
 */

public class NDimensionalData {

	private int dim;
	private int[] size;
	private int[] offset;
	private short[] data;
	
	/*
	 * pivot element is an offset of the beginning of the array!
	 */
	public NDimensionalData(int dimension, int[] data_extent, short[] data) {
		
		dim = dimension;
		size = new int[dim];
		offset = new int[dim];
		
		this.data = data;
		
		for (int i=0; i<dimension; i++) {
			size[i] = data_extent[i];
			offset[i] = 0;
		}
		
	}
	
	public void defineDataOffset(int[] data_offset) {
		offset = data_offset;
	}
	
	public short getValue(int position) {
		
		//to nDimensional
		int[] nd = LinearNDMapping.toNDimensional(dim, size, position);
		int[] tmp = new int[dim];
		//add offset
		for(int i=0; i<dim; i++) {
			tmp[i] = nd[i] - offset[i]; //VALUE BACK TO THE ARRAY
		}
		//back to linear
		int lin_index = LinearNDMapping.toLinear(dim, size, tmp);
		
		return data[lin_index];
	}
	
	public short getValue(int[] position) {
		
		for (int i=0; i<dim; i++) {
			position[i] -= offset[i];
		}
		
		/*//to linear index! //old an potentially wrong
		int lin_index = 0;
		for (int i=0; i<dim; i++) {
			
			if (position[i] >= size[i]) {
				System.out.println("Array out of bounds in get!");
				lin_index = -1;
				break;
			}
			
			int size_mult = 1;
			for (int j=0; j<i; j++) {
				size_mult *= size[j];
			}
			
			lin_index += size_mult * position[i];
		}*/
		
		int lin_index = LinearNDMapping.toLinear(dim, size, position);
		
		return data[lin_index];
		
	}
	
	public void setValue(int[] position, short val) {
		
		for (int i=0; i<dim; i++) {
			position[i] -= offset[i];
		}
		
		/*//to linear index! //old an potentially wrong
		int lin_index = 0;
		for (int i=0; i<dim; i++) {
			
			if (position[i] >= size[i]) {
				System.out.println("Array out of bounds in set!");
				lin_index = -1;
				break;
			}
			
			int size_mult = 1;
			for (int j=0; j<i; j++) {
				size_mult *= size[j];
			}
			
			lin_index += size_mult * position[i];
		}*/
		
		int lin_index = LinearNDMapping.toLinear(dim, size, position);
	
		data[lin_index] = val;
	}
	
}
