package gkdtree;

/**
 * 
 * Approximations for normal distribution.
 * 
 * @author DarkMatt
 *
 */

public class NormalDist {
	
	/**
	 * Approximation of a normal cumulative probability density function (cdf).
	 * 
	 * Source:
	 * http://www.wilmott.com/messageview.cfm?catid=4&threadid=67838
	 * https://lyle.smu.edu/~aleskovs/emis/sqc2/accuratecumnorm.pdf
	 * 
	 * @param x
	 * @return float
	 */
	
	public static float normcdf(float x) {
		
		float x_abs = Math.abs(x);
		float result = 0.0f;
		
		if (x_abs < 37) {
			float exponential = (float)Math.exp(-(x_abs*x_abs)/2.0f);
			float build = 0.0f;
			if (x_abs < 7.07106781186547f) {
				build = 3.52624965998911E-02f * x_abs + 0.700383064443688f;
				build = build * x_abs + 6.37396220353165f;
				build = build * x_abs + 33.912866078383f;
				build = build * x_abs + 112.079291497871f;
				build = build * x_abs + 221.213596169931f;
				build = build * x_abs + 220.206867912376f;
				result = exponential * build;
				build = 8.83883476483184E-02f * x_abs + 1.75566716318264f;
				build = build * x_abs + 16.064177579207f;
				build = build * x_abs + 86.7807322029461f;
				build = build * x_abs + 296.564248779674f;
				build = build * x_abs + 637.333633378831f;
				build = build * x_abs + 793.826512519948f;
				build = build * x_abs + 440.413735824752f;
				result /= build;
			} else {
				build = x_abs + 0.65f;
				build = x_abs + 4 / build;
				build = x_abs + 3 / build;
				build = x_abs + 2 / build;
				build = x_abs + 1 / build;
				result = exponential / build / 2.506628274631f;
			}
		}
		
		if (x > 0)
			return 1.0f - result;
		else
			return result;
	}
	
}

