package gkdtree;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * Provides static methods for:
 *  - converting arrays of grid structures to point lists
 *  (point lists define each coordinate value explicitly) 
 * 
 * @author DarkMatt
 *
 */

public class PointList {
	
	/*
	 * This is the killer method created at 5:15 AM.
	 * 
	 * What it does? - Converts
	 */
	public static ArrayList<Float> getArrayList(Float[] in, Boundary b, Boundary selection) {
		ArrayList<Float> out = new ArrayList<Float>(); 
		out = (ArrayList<Float>)Arrays.asList(in);
		Boundary min = b.getMinimalSubregion(selection);
		int[] pos = min.getLower();
		int[] counter = new int[min.getDimensionality()];
		for (int d=0; d<min.getDimensionality(); d++) {
			counter[d] = 0;
		}
		
		ArrayList<Float> combined = new ArrayList<Float>();
		for (int i=0; i<min.getNumberOfElements(); i++) {
			for (int d=0; d<min.getDimensionality(); d++) {
				//if overflow (last dimension overflow shouldn't happen)
				if (counter[d] >= min.getSize()[d]) {
					counter[d] = 0;
					counter[d+1] ++;
				}
				combined.add((float)(pos[d]+counter[d]));
			}
			combined.add(out.get(i));
			counter[0] ++;
		}
		return combined;
	}
	
	/**
	 * Converts array to list. 
	 * I wouldn't advise trying to understand this.
	 * 
	 * @param in
	 * @param b
	 * @param selection
	 * @return ArrayList<Float>
	 */
	public static ArrayList<Float> getArrayList(Short[] in, Boundary b, Boundary selection) {
		List<Short> out = new ArrayList<Short>();
		out = Arrays.asList(in);
		Boundary min = b.getMinimalSubregion(selection);
		int[] pos = min.getLower();
		int[] counter = new int[min.getDimensionality()];
		for (int d=0; d<min.getDimensionality(); d++) {
			counter[d] = 0;
		}
		
		//if (out.size() != min.getNumberOfElements()) {
			/*System.out.println("ERRORRRRRRRRRRRRRRRRRRR: " 
					+ out.size() + " vs " + min.getNumberOfElements() + 
					" min: " + min.getSize()[0] + "/" + min.getSize()[1] + "/" + min.getSize()[2] + 
					" b: " + b.getSize()[0] + "/" + b.getSize()[1] + "/" + b.getSize()[2] + 
					" sel: " + selection.getSize()[0] + "/" + selection.getSize()[1] + "/" + selection.getSize()[2]);
			//min.printBoundary("min");
			//b.printBoundary("b");
			//selection.printBoundary("selection");
		//}*/
		
		ArrayList<Float> combined = new ArrayList<Float>();
		for (int i=0; i<min.getNumberOfElements(); i++) {
			for (int d=0; d<min.getDimensionality(); d++) {
				//if overflow (last dimension overflow shouldn't happen)
				if (counter[d] >= min.getSize()[d]) {
					counter[d] = 0;
					counter[d+1] ++;
				}
				combined.add((float)(pos[d]+counter[d]));
			}
			combined.add((float)out.get(i));
			counter[0] ++;
		}
		
		return combined;
	}
	
	/**
	 * Returns the values for an n-dimensional data with 1 value per grid point.
	 * Takes a list (sorted by the correct order of linear indices) and returns 
	 * an array of the information.
	 * 
	 * @param in
	 * @param b
	 * @return Float[]
	 */
	public static Float[] getLinearArray(ArrayList<Float> in, Boundary b) {
		
		int dim = b.getDimensionality();
		Float[] out = new Float[b.getNumberOfElements()];
		
		for (int i=0; i<b.getNumberOfElements(); i++) {
			out[i] = in.get(dim*i + dim -1);
		}
		
		return out;
	}
	
	/**
	 * Same as getLinearArray() for Short.
	 * 
	 * @param in
	 * @param b
	 * @return Short[]
	 */
	public static Short[] getLinearArrayShort(ArrayList<Float> in, Boundary b) {
		
		int dim = b.getDimensionality()+1; //TODO check
		Short[] out = new Short[b.getNumberOfElements()];
		
		// TODO: doesnt seem very nice
		for (int i=0; i<b.getNumberOfElements(); i++) {
			out[i] = (Short)(in.get(dim*i + dim -1)).shortValue();
		}
		
		return out;
	}
	
}
