package rendertools;

import rendertools.TextRenderer;

import gkdtree.FilterParameters;
import io.UiFileList;

import java.util.ArrayList;

import org.lwjgl.opengl.GL11;
import org.lwjgl.util.glu.GLU;

import world.CameraFPS;
import world.Camera_old;
import world.SceneObject;
import world.VertexData;

/**
 *
 * The main renderer.
 * 
 * @author DarkMatt
 *
 */

public class RendererMain {

	private float sx,sy;
	private VBO vbo;
	private Textures textures;
	ShaderCollection shader;
	ArrayList<SceneObject> sceneobjects;
	CameraFPS camera;
	TextRenderer textrenderer;
	UiFileList uiFileList;
	
	int active_image = 0;
	int active_image_tmp = 0; //really bad hack :)
	
	String debug_text = "";
	float[] selection = new float[]{0.0f, 0.0f, -3.0f};
	
	float f_sigma_s, f_sigma_r;
	
	public RendererMain(float sx, float sy) {
		this.sx = sx;
		this.sy = sy;
		vbo = new VBO();
		
		textures = new Textures();
		//textures.add_Texture("IMG_1969");
		textures.add_Texture("IMG_1969_2");
		
		textrenderer = new TextRenderer("content/images/font.png", textures);
		
		shader = new ShaderCollection();
		
		uiFileList = new UiFileList(textures);
		active_image = uiFileList.getFilenameIndex("test.png") + 1;
		active_image_tmp = active_image;
		
		f_sigma_s = 0.1f;
		f_sigma_r = 0.1f;
	}
	
	public void upload_geometry(VertexData vd, ArrayList<SceneObject> so, CameraFPS c) {
		vbo.init(vd.getVertices(), vd.getTextCoords(), vd.getVertColors(), vd.getVerticesSize());
		sceneobjects = so;
		camera = c;
	}
	
	public void update_geometry(VertexData vd, int start, int size) {
		vbo.update(vd.getVertices(), vd.getVertColors(), start, size);
		active_image = active_image_tmp; //TODO: not very appropriate (for changing spacing after laoding is done)
	}
	
	public void draw_scene() {
		
		// Init OpenGL
        GL11.glMatrixMode(GL11.GL_PROJECTION);
        GL11.glLoadIdentity();
        GLU.gluPerspective(75.0f, sx/sy, 0.001f, 100.0f);
        //GL11.glOrtho(-sx, sx, -sy, sy, 1, -1);
        GL11.glMatrixMode(GL11.GL_MODELVIEW);
         
        //LINE ANTI ALIASING
        GL11.glHint(GL11.GL_LINE_SMOOTH_HINT, GL11.GL_NICEST);
        GL11.glEnable(GL11.GL_LINE_SMOOTH);
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        //LINE ANTI ALIASING END
        
        GL11.glClearColor(0.6f, 0.65f, 0.65f, 1.0f);
        GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
        
        textures.bind_texture(0);
        //shader.bind(ShaderCollection.BASIC);
        
        //camera.rotateY(0.01f);
        //sceneobjects.get(0).rotateX(0.00035f);
        //sceneobjects.get(0).rotateZ(0.0007f);
        //sceneobjects.get(1).rotateZ(0.0001f);
        for (int i=0; i<sceneobjects.size(); i++) {
        
        	if (sceneobjects.get(i).visible) {
        	
	        	//System.out.println(sceneobjects.get(i).position[0]+"/"+sceneobjects.get(i).position[1]+"/"+sceneobjects.get(i).position[2]);
	        	
	        	//TODO optimize
	        	shader.bind(sceneobjects.get(i).shader, sceneobjects.get(i).drawingmode);
	        	
	        	shader.setBasicParameters(	sceneobjects.get(i).rotation, 
	        								sceneobjects.get(i).position,
	        								camera.get_viewMatrix(),
	        								camera.get_projMatrix(),
	        								sceneobjects.get(i).color);
	        	
	        	// last both are particles
	        	if (sceneobjects.get(i).drawingmode == SceneObject.PARTICLE_R) { //TODO: general solution! - this is a fast VIS-HACK!
	        		
	        		float out = 0.0f;
	        		int[] s = textures.getSize(active_image);
	        		out = 1.5f / (float)s[0];
	        		
	        		shader.setParticleParameters(	sceneobjects.get(i).rotation, 
							sceneobjects.get(i).position,
							camera.get_viewMatrix(),
							camera.get_projMatrix(),
							selection,
							uiFileList.getMode(),
							out);//0.015f); //0.015 - 100
	        	}
	        
	        	vbo.render(	sceneobjects.get(i).getType(), 
	        				sceneobjects.get(i).startIndex,
	        				sceneobjects.get(i).endIndex);
	        	
	        	//TODO optimize
	        	shader.unbind(sceneobjects.get(i).shader);
	        	
        	}
        	
        }
        
        GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
        
        //shader.unbind(ShaderCollection.BASIC);
        
        //draw_ui();
	}
	
	public void draw_ui(String fps, boolean s_active, String sigma_s, String sigma_r) {
		
		//TODO - not the perfect place to do this!
		if (sigma_s.length() > 0 && sigma_r.length() > 0) {
			f_sigma_s = (float)Float.valueOf(sigma_s);
			f_sigma_r = (float)Float.valueOf(sigma_r);
		}
		
		GL11.glDisable(GL11.GL_DEPTH_TEST);
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		
		GL11.glMatrixMode(GL11.GL_PROJECTION);
        GL11.glLoadIdentity();
        GL11.glOrtho(-sx, sx, -sy, sy, 1, -1);
        GL11.glMatrixMode(GL11.GL_MODELVIEW);
        GL11.glLoadIdentity();
        
        GL11.glColor3f(1.0f, 1.0f, 1.0f);
        
        //temporary disabled
        /*
        //crosshair
        GL11.glBegin(GL11.GL_QUADS);
	        GL11.glVertex2f(-0.02f,  0.0015f);
	        GL11.glVertex2f( 0.02f,  0.0015f);
	        GL11.glVertex2f( 0.02f, -0.0015f);
	        GL11.glVertex2f(-0.02f,	-0.0015f);
	    GL11.glEnd();
	    GL11.glBegin(GL11.GL_QUADS);
	        GL11.glVertex2f(-0.0015f,  0.02f);
	        GL11.glVertex2f( 0.0015f,  0.02f);
	        GL11.glVertex2f( 0.0015f, -0.02f);
	        GL11.glVertex2f(-0.0015f, -0.02f);
	    GL11.glEnd();*/
		
	    textrenderer.draw_string("FPS:   " + fps, 1.0f, 0.92f, 0.3f);
	    
	    //textrenderer.draw_string(debug_text, 1.0f, 0.85f, 0.3f);
	    //textrenderer.draw_string(debug_text, 0.01f, 0.0f, 0.3f);
	    
	    if (s_active) GL11.glColor3f(1.0f, 1.0f, 1.0f);
	    else GL11.glColor3f(0.2f, 0.2f, 0.2f);
	    textrenderer.draw_string("Sigma   spatial:   " + sigma_s, -1.0f, -0.9f, 0.35f);
	    if (s_active) GL11.glColor3f(0.2f, 0.2f, 0.2f);
	    else GL11.glColor3f(1.0f, 1.0f, 1.0f);
	    textrenderer.draw_string("Sigma   range:   " + sigma_r, -0.35f, -0.9f, 0.35f);
	    
	    GL11.glColor3f(1.0f, 1.0f, 1.0f);
	    
	    //help ouput
	    textrenderer.draw_string("Display mode ... 'V'", 0.9f, -0.5f, 0.28f);
	    textrenderer.draw_string("Blur ... 'B'", 0.9f, -0.55f, 0.28f);
	    
	    uiFileList.renderList(textrenderer, textures);
	    
	    GL11.glEnable(GL11.GL_TEXTURE_2D);
	    GL11.glEnable(GL11.GL_DEPTH_TEST);
	}
	
	public void draw_loading() {
		// Init OpenGL
        GL11.glMatrixMode(GL11.GL_PROJECTION);
        GL11.glLoadIdentity();
        GL11.glOrtho(-sx, sx, -sy, sy, 1, -1);
        GL11.glMatrixMode(GL11.GL_MODELVIEW);
        
        GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);

        // Begin drawing
        /*GL11.glBegin(GL11.GL_TRIANGLES);
            // Top & Red
            GL11.glColor3f(1.0f, 0.0f, 0.0f);
            GL11.glVertex2f(-1.0f, 1.0f);

            // Right & Green
            GL11.glColor3f(0.0f, 1.0f, 0.0f);
            GL11.glVertex2f(1.0f, 1.0f);

            // Left & Blue
            GL11.glColor3f(0.0f, 0.0f, 1.0f);
            GL11.glVertex2f(1.0f, -1.0f);
        GL11.glEnd();*/
	}
	
	public void set_Selection_Pos(float[] pos) {
		selection = pos;
	}
	
	public UiFileList getUiFileList() {
		return uiFileList;
	}
	
	public FilterParameters getFilterParams() {
		active_image_tmp = uiFileList.getCurrentImgeIndex();
		return new FilterParameters(
				"data/"+uiFileList.getFilename(), 
				uiFileList.getSize(textures), 
				uiFileList.isBlurred(),
				f_sigma_s, f_sigma_r);
	}
	
	public void set_dbug_text(String text) {
		debug_text = text;
	}
	
	public void draw_debug_dir(float[] dir) {
        
		/*shader.bind(ShaderCollection.BASIC);
		
		GL11.glColor3f(0.0f, 1.0f, 0.0f);
		
		shader.setBasicParameters(	sceneobjects.get(0).rotation, 
									sceneobjects.get(0).position,
									camera.get_viewMatrix());
		
        GL11.glBegin(GL11.GL_TRIANGLES);
	        GL11.glVertex3f(dir[0]+0.2f, dir[1]+0.2f, dir[2]+0.0f);
	        GL11.glVertex3f(dir[0]+0.0f, dir[1]+0.0f, dir[2]+0.0f);
	        GL11.glVertex3f(dir[0]+dir[3], dir[1]+dir[4], dir[2]+dir[5]);
	    GL11.glEnd();
	    
	    shader.unbind(ShaderCollection.BASIC);*/
	    
	}
	
}
