package rendertools;

import java.nio.FloatBuffer;
import java.util.ArrayList;

import org.lwjgl.opengl.GL11;

import world.SceneObject;

/**
 *
 * Shader collection handles the parameters of Shaders and all the
 * parameters that have to be defined manually on a centralized fashion.
 * 
 * @author DarkMatt
 *
 */

public class ShaderCollection {

	/*
	 * Merely to integrate the available shader into the engine.
	 * 
	 * Possibility to automate this process later on.
	 */
	
	//shader
	public static final int BASIC = 0;
	public static final int PARTICLE = 1;
	
	private ArrayList<Shader> shader;
	private int basic_model_rotation, basic_model_translation, basic_viewMatrix, basic_projMatrix, basic_color;
	private int particle_model_rotation, particle_model_translation, particle_viewMatrix, particle_projMatrix, particle_selection, particle_mode, particle_scale;
	
	public ShaderCollection() {
		shader = new ArrayList<Shader>();
		
		shader.add(new Shader("/shader/default"));
		basic_model_rotation = shader.get(BASIC).initValueMat3f("model_rotation");
		basic_model_translation = shader.get(BASIC).initValueVec3f("model_translation");
		basic_viewMatrix = shader.get(BASIC).initValueMat4f("viewMatrix");
		basic_projMatrix = shader.get(BASIC).initValueMat4f("projMatrix");
		basic_color = shader.get(BASIC).initValueVec3f("color");
		
		shader.add(new Shader("/shader/particle"));
		particle_model_rotation = shader.get(PARTICLE).initValueMat3f("model_rotation");
		particle_model_translation = shader.get(PARTICLE).initValueVec3f("model_translation");
		particle_viewMatrix = shader.get(PARTICLE).initValueMat4f("viewMatrix");
		particle_projMatrix = shader.get(PARTICLE).initValueMat4f("projMat");
		particle_selection = shader.get(PARTICLE).initValueVec3f("sel_pos");
		particle_mode = shader.get(PARTICLE).initValue1f("particle_mode");
		particle_scale = shader.get(PARTICLE).initValue1f("particle_scale");
	}
	
	public void setBasicParameters(float[] model_rotation, float[] model_translation, FloatBuffer viewMatrix, FloatBuffer projMatrix, float[] color) {
		shader.get(BASIC).setValueMat3f(basic_model_rotation, model_rotation);
		shader.get(BASIC).setValueVec3f(basic_model_translation, 
				model_translation[0], model_translation[1], model_translation[2]);
		shader.get(BASIC).setValueMat4f(basic_viewMatrix, viewMatrix);
		shader.get(BASIC).setValueMat4f(basic_projMatrix, projMatrix);
		shader.get(BASIC).setValueVec3f(basic_color, color[0], color[1], color[2]);
	}
	
	public void setParticleParameters(float[] model_rotation, float[] model_translation, FloatBuffer viewMatrix, FloatBuffer projMatrix, float[] selection, float mode, float scale) {
		shader.get(PARTICLE).setValueMat3f(particle_model_rotation, model_rotation);
		shader.get(PARTICLE).setValueVec3f(particle_model_translation, 
				model_translation[0], model_translation[1], model_translation[2]);
		shader.get(PARTICLE).setValueMat4f(particle_viewMatrix, viewMatrix);
		shader.get(PARTICLE).setValueMat4f(particle_projMatrix, projMatrix);
		shader.get(PARTICLE).setValueVec3f(particle_selection, 
				selection[0], selection[1], selection[2]);
		shader.get(PARTICLE).setValue1f(particle_mode, mode);
		shader.get(PARTICLE).setValue1f(particle_scale, scale); // 0.0025f for 500x500
	}
	
	public void bind(int name, int drawingmode) {
		if (drawingmode == SceneObject.WIREFRAME) {
			GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_LINE);
		} else { //shaded + particle
			GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
		}
		shader.get(name).bind();
	}
	
	public void unbind(int name) {
		shader.get(name).unbind();
	}
	
}
