package rendertools;

import gkdtree.FileNode;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.imageio.ImageIO;

import org.lwjgl.opengl.GL11;

/**
 *
 * Basic text renderer functionality.
 * 
 * @author DarkMatt
 *
 */

public class TextRenderer {

	static final int TEXTUREINDEX = 1; //TODO!
	
	Textures textures;
	
	private static final int FIRST_ROW_OFFSET = 6;
	private static final int ROW_OFFSET = 2;
	private static final int ROWS = 6;
	String[] c = new String[]{
			"A","B","C","D","E","F","G","H","I","J","K","L","M","N", 	//14
			"O","P","Q","R","S","T","U","V","W","X","Y","Z",			//12
			"a","b","c","d","e","f","g","h","i","j","k","l","m","n",	//14
			"o","p","q","r","s","t","u","v","w","x","y","z",			//12
			"1","2","3","4","5","6","7","8","9","0",					//10
			"*","+","'","#",",",";",".",":","_","-"};					//10
	int[] per_line = new int[]{14,12,14,12,10,10};
	int[] offset; //sum - 6
	
	float[] cx1, cy1, cx2, cy2;
	
	public TextRenderer(String path, Textures text) {
		
		textures = text;
		//could be stored in a text file!
		
		int num = 0;
		for (int i=0; i<per_line.length; i++) num += per_line[i];
		
		offset = new int[num];
		
		URI uri = null;
		try {
			uri = new URI("file://"+FileNode.class.getProtectionDomain().getCodeSource().getLocation().getPath());
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
		File absolute_path = new File(new File(uri).getParent());
		
		BufferedImage img = null;
		try {
		    img = ImageIO.read(new File(absolute_path +"/"+ path));
		} catch (IOException e) {
			System.out.println(path);
			e.printStackTrace();
		}
		
		int sx = img.getWidth();
		int sy = img.getHeight();
		int c_c = 0;
		
		//find character offsets
		for (int i=0; i<ROWS; i++) {
			int start = i*sy/ROWS + ROW_OFFSET;
			int end = (i+1)*sy/ROWS - ROW_OFFSET;
			int curr_x = 0;
			
			//for every letter
			for (int j=0; j < (per_line[i]); j++) {
				boolean found = false;
				boolean space_found = false;
				while(!found && curr_x<sx) {
					int line = start;
					for (int k=start; k<end; k++) {
						if (((img.getRGB(curr_x, k) >> 24) & 0xff) >= 20) { //if alpha is not 0
							if (space_found) {
								found = true;
								offset[c_c] = curr_x;
								//System.out.println("found[" + j + "]: " + c[c_c] + " at " + curr_x); //DEBUG
								c_c ++;
							}
							break;	
						}
						line ++;
					}
					
					//System.out.println(j + ": " + line + "/" + end);
					
					if (curr_x > FIRST_ROW_OFFSET && line == end) {
						space_found = true;
						if (j == per_line[i]-1) { //last letter
							found = true;
							offset[c_c] = curr_x + 2;
							//System.out.println("last found[" + j + "]: " + c[c_c] + " at " + curr_x); //DEBUG
							c_c ++;
						}
					}
					
					curr_x ++;
				}
			}
		}
		
		cx1 = new float[c_c];
		cy1 = new float[c_c];
		cx2 = new float[c_c];
		cy2 = new float[c_c];
		
		//save character to texture mapping
		c_c = 0;
		for (int i=0; i<ROWS; i++) {
			
			int prev = 0;
			
			for (int j=0; j < (per_line[i]); j++) {
				
				int curr = offset[c_c];
				
				cx1[c_c] = ((float)prev + 0.0f)/(float)sx;
				cy1[c_c] = (float)i/((float)ROWS + 0.25f);
				cx2[c_c] = ((float)curr - 0.5f)/(float)sx;
				cy2[c_c] = (float)(i+1)/((float)ROWS + 0.25f);
				
				prev = curr;
				
				c_c ++;
				
			}
			
		}
		
		//load texture
		
		//try {
			text.add_Texture("font");
			//font_texture = TextureLoader.getTexture("PNG", ResourceLoader.getResourceAsStream("Images/font.png"));
		/*} catch (IOException e) {
			e.printStackTrace();
		}*/
		
	}
	
	public void draw_string(String text, float px, float py, float scale) {
		
		//GL11.glBindTexture(GL11.GL_TEXTURE_2D, font_texture.getTextureID());
		
		GL11.glEnable(GL11.GL_BLEND);
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
		
		textures.bind_texture(TEXTUREINDEX); //TODO: check
		
		GL11.glPushMatrix();
		
		float dx = 0.0f;
		
		for (int i=0; i<text.length(); i++) {
			
			for (int j=0; j<c.length; j++) {
				
				if (c[j].equals(text.substring(i, i+1))) {
					
					//GL11.glColor3f(1.0f, 1.0f, 1.0f);
					
					//GL11.glColor3f(1.0f, 1.0f, 1.0f); //red try
					
					GL11.glTranslatef(dx + 0.01f*scale, 0.0f, 0.0f);
					
					//System.out.println(cx1[j] +"/"+ cy1[j] +"/"+ cx2[j] +""+ cy2[j]);
					
					dx = cx2[j] - cx1[j];
					float dy = cy2[j] - cy1[j];
					
					dx *= scale;
					dy *= scale;
					
					GL11.glBegin(GL11.GL_QUADS);
					GL11.glTexCoord2f(cx1[j], cy2[j]);
					GL11.glVertex2f(px, py);
					GL11.glTexCoord2f(cx2[j], cy2[j]);
					GL11.glVertex2f(px+dx, py);
					GL11.glTexCoord2f(cx2[j], cy1[j]); 
					GL11.glVertex2f(px+dx, py+dy);
					GL11.glTexCoord2f(cx1[j], cy1[j]);
					GL11.glVertex2f(px, py+dy);
					GL11.glEnd();
					
					
				} else if (text.substring(i, i+1).equals(" ")){
					GL11.glTranslatef(0.0002f * scale, 0.0f, 0.0f);
				}
				
			}
			
		}
		
		GL11.glPopMatrix();
		
		GL11.glDisable(GL11.GL_BLEND);
		
	}
	
}
