package rendertools;

import java.io.IOException;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.ArrayList;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;
import org.newdawn.slick.opengl.Texture;
import org.newdawn.slick.opengl.TextureLoader;
import org.newdawn.slick.util.ResourceLoader;

/**
 *
 * Texture load, init, bind abstraction.
 * 
 * @author DarkMatt
 *
 */

public class Textures {

	//TODO: index 0 for terrain
	//TODO: other index for textured objects
	
	private int current_index;
	private IntBuffer textures; //directly read textures
	private ArrayList<Texture> texture_list;
	
	public Textures() {
		
		textures = BufferUtils.createIntBuffer(10);
		GL11.glGenTextures(textures);
		
		texture_list = new ArrayList<Texture>();
		
	}
	
	public int add_Texture(String name) {
		return add_Texture_At_Path("content/images/"+name+".png");
	}
	
	public int add_Texture_Full_Name(String name) {
		return add_Texture_At_Path("data/"+name);
	}
	
	private int add_Texture_At_Path(String name) {
		try {
			
			
			texture_list.add(
					TextureLoader.getTexture(
							"PNG", 
							ResourceLoader.getResourceAsStream(name),
							GL11.GL_LINEAR)
			);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL11.GL_REPEAT);
			GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL11.GL_REPEAT);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		current_index++;
		
		return current_index;
	}
	
	public int add_Texture(float[] data, int sx, int sy) {
		
		FloatBuffer tex_data = BufferUtils.createFloatBuffer(3*2*sx*sy); //why 4*2?
		tex_data.put(data);
		tex_data.position(0);
		
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, textures.get(0));
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR);
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL11.GL_REPEAT);
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL11.GL_REPEAT);
		GL11.glTexImage2D(GL11.GL_TEXTURE_2D, 0, 
				GL11.GL_RGB, sx, sy, 
				0, GL11.GL_RGB, GL11.GL_FLOAT, tex_data);
		
		current_index++;
		
		return current_index;
	}
	
	public void bind_texture(int index) {
		//if (index == 0)
		//	GL11.glBindTexture(GL11.GL_TEXTURE_2D, textures.get(index));
		//else
		texture_list.get(index).bind();
	}
	
	public float[] getOffset(int i) {
		float[] out = new float[2];
		out[0] = texture_list.get(i).getWidth();
		out[1] = texture_list.get(i).getHeight();
		
		//System.out.println(texture_list.get(i).getImageWidth() + "/" + texture_list.get(i).getWidth());
		
		return out;
	}
	
	public int[] getSize(int i) {
		int[] out = new int[2];
		out[0] = texture_list.get(i).getImageWidth();
		out[1] = texture_list.get(i).getImageHeight();
		return out;
	}
	
}
