package tests;

import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;
import org.lwjgl.util.vector.Vector4f;

public class MatrixMultiplication {

	public class MultThread extends Thread {
		
		private float[] transform, vertices;
		int size;
		
		public MultThread(float[] transform, float[] vertices, int size) {
			this.transform = transform;
			this.vertices = vertices;
			this.size = size;
		}
		
		private void mult_custom(float[] transform, float[] vertices, int i) {
			float x = vertices[4*i];
			float y = vertices[4*i+1];
			float z = vertices[4*i+2];
			float w = vertices[4*i+3];
			
			vertices[4*i] = 
					transform[0]*x + 
					transform[1]*y +
					transform[2]*z +
					transform[3]*w;
			vertices[4*i+1] = 
					transform[4]*x + 
					transform[5]*y +
					transform[6]*z +
					transform[7]*w;
			vertices[4*i+2] = 
					transform[8]*x + 
					transform[9]*y +
					transform[10]*z +
					transform[11]*w;
			vertices[4*i+3] = 
					transform[12]*x + 
					transform[13]*y +
					transform[14]*z +
					transform[15]*w;
		}
		
		public void run() {
			for (int i=0; i<size; i++) {
				mult_custom(transform, vertices, i);
			}
			
			//System.out.println("Thread done.");
		}
		
	}
	
	public MatrixMultiplication() {
		
		//execute_jblas_mult();
		
		//execute_lwjgl_mult();
		
		execute_selfmade_mult();
		
		execute_selfmade_multicore_mult();
		
	}
	
	private void execute_jblas_mult() {
		
		FloatMatrix[] vertices, vertices_out;
		
		final int size = 1000000; //1 mil
		
		FloatMatrix transform = 
			new FloatMatrix(new float[][]{
                {1.0f, 0.0f, 3.0f, 0.0f},
                {0.0f, 1.0f, 0.0f, 0.0f},
                {7.0f, 0.0f, 9.0f, 0.0f},
                {0.0f, 0.0f, 0.0f, 1.0f}
            });
		
		vertices = new FloatMatrix[size];
		vertices_out = new FloatMatrix[size];
		
		float mod = 0.0f;
		for (int i=0; i<size; i++) {
			vertices[i] = new FloatMatrix(new float[] {1.0f+mod, 1.0f-mod, 1.0f, 1.0f});
			mod += 0.001f;
			vertices_out[i] = new FloatMatrix(4);
		}
		
		System.out.println("Starting JBLAS multiplication...");
		
		long time = System.currentTimeMillis();
		
		for (int j=0; j<1; j++) {
			for (int i=0; i<size; i++) {
				 transform.mmuli(vertices[i], vertices_out[i]);
			}
		}
		
		float total = (float)(System.currentTimeMillis()-time)/1000.0f;
		
		System.out.println("End: " + total);
		
	}
	
	private void execute_lwjgl_mult() {
		
		Vector4f[] vertices;//, vertices_out;
		
		final int size = 10000000; //1 mil
		
		Matrix4f transform = new Matrix4f();
		//transform.rotate(66, new Vector3f(1.0f, 0.0f, 0.0f));
		
		vertices = new Vector4f[size];
		//vertices_out = new Vector4f[size];
		
		float mod = 0.0f;
		for (int i=0; i<size; i++) {
			vertices[i] = new Vector4f(1.0f+mod, 1.0f-mod, 1.0f, 1.0f);
			mod += 0.001f;
			//vertices_out[i] = new Vector4f();
		}
		
		System.out.println("Starting LWJGL multiplication...");
		
		long time = System.currentTimeMillis();
		
		for (int j=0; j<10; j++) {
			for (int i=0; i<size; i++) {
				Matrix4f.transform(transform, vertices[i], vertices[i]);
			}
		}
		
		//System.out.println(vertices_out[100].x+vertices_out[1000].x);
		
		float total = (float)(System.currentTimeMillis()-time)/1000.0f;
		
		System.out.println("End: " + total);
		
		/*
		 * Results: 1 mil times => ~18ms <=> ~60 fps
		 */
		
	}
	
	
	
	private void execute_selfmade_multicore_mult() {
		
		float[] vertices1, vertices2;//, vertices_out;
		
		final int size = 20000000; //1 mil
		
		
		float[] transform = 
				new float[]{
					1.0f, 0.0f, 0.0f, 0.0f,
					0.0f, 1.0f, 0.0f, 0.0f,
					0.0f, 0.0f, 1.0f, 0.0f,
					0.0f, 0.0f, 0.0f, 1.0f
				};
		
		vertices1 = new float[4*size];
		vertices2 = new float[4*size];
		
		float mod = 0.0f;
		for (int i=0; i<size; i++) {
			vertices1[4*i] = 1.0f+mod;
			vertices1[4*i+1] = 1.0f-mod;
			vertices1[4*i+2] = 1.0f;
			vertices1[4*i+3] = 1.0f;
			mod += 0.001f;
		}
		for (int i=0; i<size; i++) {
			vertices2[4*i] = 1.0f+mod;
			vertices2[4*i+1] = 1.0f-mod;
			vertices2[4*i+2] = 1.0f;
			vertices2[4*i+3] = 1.0f;
			mod += 0.001f;
		}
		
		System.out.println("Starting CUSTOM MULTICORE multiplication...");
		
		long time = System.currentTimeMillis();
		
		MultThread m1 = new MultThread(transform, vertices1, size);
		MultThread m2 = new MultThread(transform, vertices2, size);
		
		m1.start();
		m2.start();
		
		try {
			m1.join();
			m2.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		
		//System.out.println(vertices_out[100].x+vertices_out[1000].x);
		
		float total = (float)(System.currentTimeMillis()-time)/1000.0f;
		
		System.out.println("End: " + total);
		
	}
	
	private void execute_selfmade_mult() {
		
		float[] vertices;//, vertices_out;
		
		final int size = 40000000; //1 mil
		
		
		float[] transform = 
				new float[]{
					1.0f, 0.0f, 0.0f, 0.0f,
					0.0f, 1.0f, 0.0f, 0.0f,
					0.0f, 0.0f, 1.0f, 0.0f,
					0.0f, 0.0f, 0.0f, 1.0f
				};
		
		vertices = new float[4*size];
		//vertices_out = new float[4*size];
		
		float mod = 0.0f;
		for (int i=0; i<size; i++) {
			vertices[4*i] = 1.0f+mod;
			vertices[4*i+1] = 1.0f-mod;
			vertices[4*i+2] = 1.0f;
			vertices[4*i+3] = 1.0f;
			mod += 0.001f;
			//vertices_out[4*i] = 1.0f;
			//vertices_out[4*i+1] = 1.0f;
			//vertices_out[4*i+2] = 1.0f;
			//vertices_out[4*i+3] = 1.0f;;
		}
		
		System.out.println("Starting CUSTOM multiplication...");
		
		long time = System.currentTimeMillis();
		
		for (int j=0; j<1; j++) {
			
			for (int i=0; i<size; i++) {
				
				float x = vertices[4*i];
				float y = vertices[4*i+1];
				float z = vertices[4*i+2];
				float w = vertices[4*i+3];
				
				vertices[4*i] = 
						transform[0]*x + 
						transform[1]*y +
						transform[2]*z +
						transform[3]*w;
				vertices[4*i+1] = 
						transform[4]*x + 
						transform[5]*y +
						transform[6]*z +
						transform[7]*w;
				vertices[4*i+2] = 
						transform[8]*x + 
						transform[9]*y +
						transform[10]*z +
						transform[11]*w;
				vertices[4*i+3] = 
						transform[12]*x + 
						transform[13]*y +
						transform[14]*z +
						transform[15]*w;
				//mult_custom(transform, vertices, i);
			}
		}
		
		//System.out.println(vertices_out[100].x+vertices_out[1000].x);
		
		float total = (float)(System.currentTimeMillis()-time)/1000.0f;
		
		System.out.println("End: " + total);
		
	}
	
}
