package world;

import java.nio.FloatBuffer;

import org.lwjgl.BufferUtils;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

/**
 *
 * First person camera implementation.
 * 
 * @author DarkMatt
 * 
 */

public class CameraFPS {

	private Vector3f pos;
	private float rot_x, rot_y;
	private Matrix4f cameraMatrix, viewMatrix, projMatrix; //rotate 3f not implemented!
	
	/*
	 * Explanation:
	 *  - rotation y first
	 */
	
	public CameraFPS(float sx, float sy) {
		pos = new Vector3f(0.0f, 0.0f, 0.0f);
		rot_x = 0.0f;
		rot_y = 0.0f;
		cameraMatrix = new Matrix4f();
		viewMatrix = new Matrix4f();
		projMatrix = new Matrix4f();
		
		set_projection(60.0f, sx/sy, 0.001f, 100.0f); //75.0
		//set_projection(30.0f, sx/sy, 0.001f, 100.0f);
		//set_ortho(sx, sy, 0.001f, 100.0f);
	}
	
	//based on gluPersective
	public void set_projection(float fovy, float aspect, float zNear, float zFar) {
		
		float f = 1.0f / (float)Math.tan(Math.toRadians(fovy)/2.0f);
		
		projMatrix.m00 = f/aspect;
		projMatrix.m11 = f;
		projMatrix.m22 = (zFar+zNear)/(zNear-zFar);
		projMatrix.m32 = (2*zFar*zNear)/(zNear-zFar);
		projMatrix.m23 = -1.0f;
		projMatrix.m33 = 0.0f;
		
	}
	
	//based on gluPersective
	public void set_ortho(float w, float h, float zNear, float zFar) {
		
		projMatrix.m00 = 2/w*400;
		projMatrix.m11 = 2/h*400;
		projMatrix.m22 = 1/(zNear-zFar);
		projMatrix.m32 = -zNear/(zNear-zFar);
		projMatrix.m33 = 1.0f;
		
	}
	
	//wrt y-rotation
	public void translate(float x, float y, float z) {
		//straight forward solution
		Matrix4f temp = new Matrix4f();
		Matrix4f.translate(pos, temp, temp);
		Matrix4f.rotate(rot_y, new Vector3f(0.0f, 1.0f, 0.0f), temp, temp);
		Matrix4f.translate(new Vector3f(x, y, z), temp, temp);
		pos.x = temp.m30;
		pos.y = temp.m31;
		pos.z = temp.m32;
		build_matrices();
	}
	
	public void rotX(float a) {
		rot_x += a;
		
		if (rot_x < -(float)Math.PI)	rot_x += 2.0f*(float)Math.PI;
		if (rot_x > (float)Math.PI)		rot_x -= 2.0f*(float)Math.PI;
		
		build_matrices();
	}
	
	public void rotY(float a) {
		rot_y += a;
		
		if (rot_y < -(float)Math.PI)	rot_y += 2.0f*(float)Math.PI;
		if (rot_y > (float)Math.PI)		rot_y -= 2.0f*(float)Math.PI;
		
		build_matrices();
	}
	
	private void build_matrices() {
		cameraMatrix = Matrix4f.setIdentity(cameraMatrix);
		Matrix4f.translate(pos, cameraMatrix, cameraMatrix);
		Matrix4f.rotate(rot_y, new Vector3f(0.0f, 1.0f, 0.0f), cameraMatrix, cameraMatrix);
		Matrix4f.rotate(rot_x, new Vector3f(1.0f, 0.0f, 0.0f), cameraMatrix, cameraMatrix);
		
		viewMatrix = Matrix4f.setIdentity(viewMatrix);
		Matrix4f.rotate(-rot_x, new Vector3f(1.0f, 0.0f, 0.0f), viewMatrix, viewMatrix);
		Matrix4f.rotate(-rot_y, new Vector3f(0.0f, 1.0f, 0.0f), viewMatrix, viewMatrix);
		Matrix4f.translate(new Vector3f(-pos.x, -pos.y, -pos.z), viewMatrix, viewMatrix);
		
		//DEBUG
		/*
		Matrix4f debug = new Matrix4f();
		Matrix4f.mul(viewMatrix, cameraMatrix, debug);
		System.out.println(debug.m00 + " " + debug.m10 + " " + debug.m20 + " " + debug.m30);
		System.out.println(debug.m01 + " " + debug.m11 + " " + debug.m21 + " " + debug.m31);
		System.out.println(debug.m02 + " " + debug.m12 + " " + debug.m22 + " " + debug.m32);
		System.out.println(debug.m03 + " " + debug.m13 + " " + debug.m23 + " " + debug.m33);
		System.out.println();
		*/
	}
	
	//Inverse camera matrix
	public FloatBuffer get_viewMatrix() {
		FloatBuffer out = BufferUtils.createFloatBuffer(16);
		out.clear();
		out.put(viewMatrix.m00); out.put(viewMatrix.m10); out.put(viewMatrix.m20); out.put(viewMatrix.m30);
		out.put(viewMatrix.m01); out.put(viewMatrix.m11); out.put(viewMatrix.m21); out.put(viewMatrix.m31);
		out.put(viewMatrix.m02); out.put(viewMatrix.m12); out.put(viewMatrix.m22); out.put(viewMatrix.m32);
		out.put(viewMatrix.m03); out.put(viewMatrix.m13); out.put(viewMatrix.m23); out.put(viewMatrix.m33);
		out.flip();
		return out;
	}
	
	//Inverse camera matrix
	public FloatBuffer get_viewMatrix4x3() {
		FloatBuffer out = BufferUtils.createFloatBuffer(12);
		out.clear();
		out.put(viewMatrix.m00); out.put(viewMatrix.m10); out.put(viewMatrix.m20); out.put(viewMatrix.m30);
		out.put(viewMatrix.m01); out.put(viewMatrix.m11); out.put(viewMatrix.m21); out.put(viewMatrix.m31);
		out.put(viewMatrix.m02); out.put(viewMatrix.m12); out.put(viewMatrix.m22); out.put(viewMatrix.m32);
		out.flip();
		return out;
	}
	
	//TODO
	/*
	public FloatBuffer get_cameraMatrix() {
		FloatBuffer out = BufferUtils.createFloatBuffer(12);
		out.clear();
		out.put(cameraMatrix.m00); out.put(cameraMatrix.m10); out.put(cameraMatrix.m20); out.put(cameraMatrix.m30);
		out.put(cameraMatrix.m01); out.put(cameraMatrix.m11); out.put(cameraMatrix.m21); out.put(cameraMatrix.m31);
		out.put(cameraMatrix.m02); out.put(cameraMatrix.m12); out.put(cameraMatrix.m22); out.put(cameraMatrix.m32);
		out.flip();
		return out;
	}
	*/
	
	public FloatBuffer get_projMatrix() {
		FloatBuffer out = BufferUtils.createFloatBuffer(16);
		out.clear();
		out.put(projMatrix.m00); out.put(projMatrix.m10); out.put(projMatrix.m20); out.put(projMatrix.m30);
		out.put(projMatrix.m01); out.put(projMatrix.m11); out.put(projMatrix.m21); out.put(projMatrix.m31);
		out.put(projMatrix.m02); out.put(projMatrix.m12); out.put(projMatrix.m22); out.put(projMatrix.m32);
		out.put(projMatrix.m03); out.put(projMatrix.m13); out.put(projMatrix.m23); out.put(projMatrix.m33);
		out.flip();
		return out;
	}
	
	public float[] get_position() {
		return new float[]{cameraMatrix.m30, cameraMatrix.m31, cameraMatrix.m32};
	}
	
	public float[] get_direction() {
		return new float[]{-cameraMatrix.m20, -cameraMatrix.m21, -cameraMatrix.m22};
	}
	
}
