package world;

public class Camera_old {
	
	private float[] position_float;
	private float[] rotation_float;
	private float[] rotation_aux;
	
	public Camera_old() {
		position_float = new float[] {0.0f, 0.0f, 0.0f};
		rotation_float = new float[] {1.0f, 0.0f, 0.0f,
								0.0f, 1.0f, 0.0f,
								0.0f, 0.0f, 1.0f};
		rotation_aux = new float[] {1.0f, 0.0f, 0.0f,
									0.0f, 1.0f, 0.0f,
									0.0f, 0.0f, 1.0f};
	}
	
	public float[] getTranslation() {
		float[] out = new float[3];
		
		out[0] = -position_float[0];
		out[1] = -position_float[1];
		out[2] = -position_float[2];
		
		return out;
	}
	
	public float[] getInverseTranslation() {
		float[] out = new float[3];
		
		out[0] = rotation_float[0]*position_float[0]+rotation_float[1]*position_float[1]+rotation_float[2]*position_float[2];
		out[1] = rotation_float[3]*position_float[0]+rotation_float[4]*position_float[1]+rotation_float[5]*position_float[2];
		out[2] = rotation_float[6]*position_float[0]+rotation_float[7]*position_float[1]+rotation_float[8]*position_float[2];
		
		return out;
	}
	
	//C: R2 * R1 * T
	//C^-1: T^-1 * R1^t * R2^t = T^-1 * (R1 * R2)^t
	//T^-1 depends on both R
	public float[] getInverseTranslation_with_aux() {
		float[] out = new float[3];
		float[] r = getInverseRotation_with_aux();
		
		out[0] = r[0]*position_float[0]+r[3]*position_float[1]+r[6]*position_float[2];
		out[1] = r[1]*position_float[0]+r[4]*position_float[1]+r[7]*position_float[2];
		out[2] = r[2]*position_float[0]+r[5]*position_float[1]+r[8]*position_float[2];
		
		return out;
	}
	
	public float[] getRotation() {
		float[] out = new float[9];
		
		out[0] = rotation_float[0]; out[1] = rotation_float[1]; out[2] = rotation_float[2];
		out[3] = rotation_float[3]; out[4] = rotation_float[4]; out[5] = rotation_float[5];
		out[6] = rotation_float[6]; out[7] = rotation_float[7]; out[8] = rotation_float[8];
		
		return out;
	}
	
	public float[] getInverseRotation() {
		float[] out = new float[9];
		
		out[0] = rotation_float[0]; out[1] = rotation_float[3]; out[2] = rotation_float[6];
		out[3] = rotation_float[1]; out[4] = rotation_float[4]; out[5] = rotation_float[7];
		out[6] = rotation_float[2]; out[7] = rotation_float[5]; out[8] = rotation_float[8];
		
		return out;
	}
	
	//C: R2 * R1 * T
	public float[] getRotation_with_aux() {
		float[] out = new float[9];
		
		out[0] = rotation_float[0]*rotation_aux[0]+rotation_float[3]*rotation_aux[1]+rotation_float[6]*rotation_aux[2];
		out[1] = rotation_float[1]*rotation_aux[0]+rotation_float[4]*rotation_aux[1]+rotation_float[7]*rotation_aux[2];
		out[2] = rotation_float[2]*rotation_aux[0]+rotation_float[5]*rotation_aux[1]+rotation_float[8]*rotation_aux[2];
		
		out[3] = rotation_float[0]*rotation_aux[3]+rotation_float[3]*rotation_aux[4]+rotation_float[6]*rotation_aux[5];
		out[4] = rotation_float[1]*rotation_aux[3]+rotation_float[4]*rotation_aux[4]+rotation_float[7]*rotation_aux[5];
		out[5] = rotation_float[2]*rotation_aux[3]+rotation_float[5]*rotation_aux[4]+rotation_float[8]*rotation_aux[5];
		
		out[6] = rotation_float[0]*rotation_aux[6]+rotation_float[3]*rotation_aux[7]+rotation_float[6]*rotation_aux[8];
		out[7] = rotation_float[1]*rotation_aux[6]+rotation_float[4]*rotation_aux[7]+rotation_float[7]*rotation_aux[8];
		out[8] = rotation_float[2]*rotation_aux[6]+rotation_float[5]*rotation_aux[7]+rotation_float[8]*rotation_aux[8];
		
		return out;
	}
	
	//C: R2 * R1 * T
	//C^-1: T^-1 * R1^t * R2^t = T^-1 * (R1 * R2)^t
	public float[] getInverseRotation_with_aux() {
		float[] out = new float[9];
		
		out[0] = rotation_float[0]*rotation_aux[0]+rotation_float[3]*rotation_aux[1]+rotation_float[6]*rotation_aux[2];
		out[3] = rotation_float[1]*rotation_aux[0]+rotation_float[4]*rotation_aux[1]+rotation_float[7]*rotation_aux[2];
		out[6] = rotation_float[2]*rotation_aux[0]+rotation_float[5]*rotation_aux[1]+rotation_float[8]*rotation_aux[2];
		
		out[1] = rotation_float[0]*rotation_aux[3]+rotation_float[3]*rotation_aux[4]+rotation_float[6]*rotation_aux[5];
		out[4] = rotation_float[1]*rotation_aux[3]+rotation_float[4]*rotation_aux[4]+rotation_float[7]*rotation_aux[5];
		out[7] = rotation_float[2]*rotation_aux[3]+rotation_float[5]*rotation_aux[4]+rotation_float[8]*rotation_aux[5];
		
		out[2] = rotation_float[0]*rotation_aux[6]+rotation_float[3]*rotation_aux[7]+rotation_float[6]*rotation_aux[8];
		out[5] = rotation_float[1]*rotation_aux[6]+rotation_float[4]*rotation_aux[7]+rotation_float[7]*rotation_aux[8];
		out[8] = rotation_float[2]*rotation_aux[6]+rotation_float[5]*rotation_aux[7]+rotation_float[8]*rotation_aux[8];
		
		return out;
	}
	
	public void translate(float x, float y, float z) {
		position_float[0] += x;
		position_float[1] += y;
		position_float[2] += z;
	}
	
	public void translate_wrt_rotation(float x, float y, float z) {
		//translate in direction of basis vectors
		//basis vectors are row vectors
		position_float[0] += rotation_float[0]*x + rotation_float[3]*y + rotation_float[6]*z;
		position_float[1] += rotation_float[1]*x + rotation_float[4]*y + rotation_float[7]*z;
		position_float[2] += rotation_float[2]*x + rotation_float[5]*y + rotation_float[8]*z;
	}
	
	//1		0		0
	//0		cos		-sin
	//0		sin		cos
	//intuitively: the 3 rows correspond to the coordinate axes
	//a rotation around the x axis keeps it in place
	public void rotateX(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
		
		float[] r = new float[9];
		r[0]=rotation_float[0]; 					r[1]=rotation_float[1]; 					r[2]=rotation_float[2];
		r[3]=c*rotation_float[3]-s*rotation_float[6];	r[4]=c*rotation_float[4]-s*rotation_float[7]; 	r[5]=c*rotation_float[5]-s*rotation_float[8];
		r[6]=s*rotation_float[3]+c*rotation_float[6];	r[7]=s*rotation_float[4]+c*rotation_float[7]; 	r[8]=s*rotation_float[5]+c*rotation_float[8];
		
		for (int i=0; i<9; i++) rotation_float[i] = r[i];
	}
	
	public void rotateX_aux(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
		
		float[] r = new float[9];
		r[0]=rotation_aux[0]; 						r[1]=rotation_aux[1]; 						r[2]=rotation_aux[2];
		r[3]=c*rotation_aux[3]-s*rotation_aux[6];	r[4]=c*rotation_aux[4]-s*rotation_aux[7]; 	r[5]=c*rotation_aux[5]-s*rotation_aux[8];
		r[6]=s*rotation_aux[3]+c*rotation_aux[6];	r[7]=s*rotation_aux[4]+c*rotation_aux[7]; 	r[8]=s*rotation_aux[5]+c*rotation_aux[8];
		
		for (int i=0; i<9; i++) rotation_aux[i] = r[i];
	}
	
	//cos	0		sin
	//0		1		0
	//-sin	0		cos
	public void rotateY(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
			
		float[] r = new float[9];
		r[0]=c*rotation_float[0]+s*rotation_float[6]; 	r[1]=c*rotation_float[1]+s*rotation_float[7]; 	r[2]=c*rotation_float[2]+s*rotation_float[8];
		r[3]=rotation_float[3];					r[4]=rotation_float[4]; 					r[5]=rotation_float[5];
		r[6]=-s*rotation_float[0]+c*rotation_float[6];	r[7]=-s*rotation_float[1]+c*rotation_float[7]; 	r[8]=-s*rotation_float[2]+c*rotation_float[8];
			
		for (int i=0; i<9; i++) rotation_float[i] = r[i];
	}
	
	//cos	-sin	0
	//sin	cos		0
	//0		0		1
	public void rotateZ(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
				
		float[] r = new float[9];
		r[0]=c*rotation_float[0]-s*rotation_float[3]; 	r[1]=c*rotation_float[1]-s*rotation_float[4]; 	r[2]=c*rotation_float[2]-s*rotation_float[5];
		r[3]=s*rotation_float[0]+c*rotation_float[3];	r[4]=s*rotation_float[1]+c*rotation_float[4]; 	r[5]=s*rotation_float[2]+c*rotation_float[5];
		r[6]=rotation_float[6];					r[7]=rotation_float[7]; 					r[8]=rotation_float[8];
				
		for (int i=0; i<9; i++) rotation_float[i] = r[i];
	}
	
}
