package world;

import gkdtree.FilterMain;
import gkdtree.FilterParameters;

import java.util.ArrayList;

import org.lwjgl.opengl.GL11;


import rendertools.ShaderCollection;


import loader.ObjectLoader;
import main.Entrance;

/**
 *
 * Main thread for scene handling. Does the loading and generation of the scene.
 * Has to handle updates in the scene at a later point.
 * 
 * @author DarkMatt
 *
 */

public class Scene extends Thread {

	/*
	 * A centralized scene representation is necessary. For collision detection etc.
	 * 
	 * Per vertex transformations on the CPU.
	 * Send the vertices to the graphics card.
	 * 
	 * Possibility to send the scene only once. Do collisions based on bounding spheres.
	 * Possibility to send the scene only once and re-send animated objects.
	 */
	
	VertexData vd;
	ArrayList<SceneObject> objects;
	CameraFPS c;
	
	public boolean was_Load = false;
	private boolean active_loading = false;
	private boolean first_points_load = true;
	private int points_index;
	
	private int previous_size;
	
	/**
	 * filter parameters
	 */
	private String f_name;
	private int[] f_size;
	private boolean f_filter;
	private float f_sig_s, f_sig_r;
	
	/**
	 * 
	 * @param sx
	 * @param sy
	 */
	
	public Scene(float sx, float sy) {
		
		vd = new VertexData();
		
		objects = new ArrayList<SceneObject>();
		
		c = new CameraFPS(sx, sy);
		
	}
	
	public void loadFilterInput(String filename, int[] size, boolean filter, float sig_s, float sig_r) {
		
		//ObjectLoader o;
		SceneObject s;
		
		previous_size = vd.getVerticesSize();
		//TODO: could be moved somewhere else... (kinda hack'y)
		FilterMain f = new FilterMain(filename, size); //shelter_5
		f.initTree(sig_s);
		
		// --- leaves (pre)
		previous_size = vd.getVerticesSize();
		float[] leaves = f.getTreeLeaves();
		float[] leaves_color = f.getTreeLeavesColor();
		//System.out.println("leaves size: " + leaves.length/3);
		//for (int i=0; i<leaves.length; i++) {
			//System.out.println("Error: " +leaves[i]);
		//}
		vd.addObject(leaves, leaves, leaves_color, leaves.length/3, false);
		s = new SceneObject(GL11.GL_POINTS, SceneObject.PARTICLE_R, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.PARTICLE;
		s.translate(-1.0f, -1.0f, -4.0f);
		s.scale(2.0f);
		s.visible = false;
		objects.add(s);
		
		/**
		 * Checker: 
		 *  - 1.0f only displays the black ones (others infinity)
		 *  - 0.001 almost all 0
		 *  - 0.01 between 0 and infinity...
		 */
		if (filter) {
			f.apply_Bilateral_Filter(sig_s, sig_r); //0.05f, 0.4f
		}
		
		/*float[] nerv = f.getKDDrawing();
		System.out.println("length: " + f.getKDDrawing().length);
		for (int i=0; i<nerv.length; i++) {
			System.out.println(nerv[i]);
		}*/
		
		// --- display tree
		previous_size = vd.getVerticesSize();
		vd.addObject(	f.getKDDrawing(), 
						f.getKDDrawingUV(), 
						f.getKDDrawingColor(),
						f.getKDDrawing().length/3, 
						false);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.color = new float[]{0.4f, 0.8f, 0.8f};
		//s.translate(-20.0f, -20.0f, -20.0f);
		//s.scale(-1.0f);
		//s.rotateY(-(float)Math.PI/2);
		s.translate(-1.0f, -1.0f, -4.0f);
		s.scale(2);
		s.visible = false;
		objects.add(s);
		
		// --- the data points!
		// possibility to select sub-regions
		//f.setSelection(new Boundary(new int[]{30, 30, 30}, new int[]{30, 20, 1}));*/
		float[] data = f.getPoints();
		int length = data.length/f.dimension;
		
		System.out.println("size: " + length);
		
		//if (length > 500*500) length = 500*500;
		
		//up to ~ 500*500
		float[] v_data = new float[3*600*500]; //10*3*length
		float[] t_data = new float[3*600*500];
		float[] c_data = new float[3*600*500];
		for (int i=0; i<length; i++) {
			v_data[3*i] = data[f.dimension*i];
			v_data[3*i+1] = data[f.dimension*i+1];
			v_data[3*i+2] = data[f.dimension*i+2];
			t_data[2*i] = data[f.dimension*i+ f.dimension-1]; //TODO
			t_data[2*i+1] = data[f.dimension*i+ f.dimension-1];
			c_data[3*i] = data[f.dimension*i+ f.dimension-1]; //TODO
			c_data[3*i+1] = data[f.dimension*i+ f.dimension-1];
			c_data[3*i+2] = data[f.dimension*i+ f.dimension-1];
			
			//System.out.println(data[f.dimension*i+ f.dimension-1]);
		}
		
		previous_size = vd.getVerticesSize();
		if (first_points_load) {
			vd.point_data[0] = previous_size;
			points_index = objects.size();
			
			vd.addObject(v_data, t_data, c_data, length, false);
			s = new SceneObject(
					GL11.GL_POINTS, SceneObject.PARTICLE_R, 
					previous_size, vd.getVerticesSize());
		} else {
			vd.overwriteObject(v_data, t_data, c_data, vd.point_data[0], length, false);
			s = new SceneObject(
					GL11.GL_POINTS, SceneObject.PARTICLE_R,  
					vd.point_data[0], vd.point_data[0]+length); //vd.point_data[0]+vd.point_data[1]
		}
		s.shader = ShaderCollection.PARTICLE;
		s.translate(-1.0f, 1.0f, -4.0f);
		s.rotateY((float)Math.PI);
		s.visible = true;
		s.scale(-2);
		//if (first_points_load) s.visible = false;
		//else s.visible = true;
		if (!first_points_load) {
			//objects.get(points_index).visible = false;
			objects.set(points_index, s);
		} else {
			objects.add(s);
		}
		if (!first_points_load) {
			objects.get(points_index).apply_object_transformation(vd);
		} else {
			previous_size = vd.getVerticesSize();
			//previous_size += 500*500;
		}
		
		vd.point_data[1] = length;
		// --- leaves (post)
		if (first_points_load) {
			
			first_points_load = false;
		}
		leaves = f.getTreeLeaves();
		leaves_color = f.getTreeLeavesColor();
		//System.out.println("leaves size: " + leaves.length/3);
		//for (int i=0; i<leaves.length; i++) {
			//System.out.println("Error: " +leaves[i]);
		//}
		vd.addObject(leaves, leaves, leaves_color, leaves.length/3, false);
		s = new SceneObject(GL11.GL_POINTS, SceneObject.PARTICLE_R, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.PARTICLE;
		s.translate(-1.0f, -1.0f, -4.0f);
		s.scale(2.0f);
		s.visible = false;
		objects.add(s);
		
	}
	
	private void loadScene() {
		//loading objects (initial scene)
		
		previous_size = 0;
		ObjectLoader o;
		SceneObject s;
		
		//Comment: Fast vis hacks...
		
		//Plane 1
		/*ObjectLoader o = new ObjectLoader("plane");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.SHADED, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.translate(0.0f, 0.0f, -2.0f);
		objects.add(s);
		/*
		//Plane 2
		o = new ObjectLoader("plane");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.SHADED, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.rotateX((float)-Math.PI/2.0f);
		s.translate(0.0f, -0.666f, -2.0f+0.666f);
		objects.add(s);
		
		/*o = new ObjectLoader("random_points");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), o.getSize(), false);
		s = new SceneObject(GL11.GL_POINTS, SceneObject.SHADED, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.PARTICLE;
		//s.translate(-20.0f, -20.0f, -20.0f);
		//s.scale(40);
		objects.add(s);*/
		
		//KDTree kdt = new KDTree(3, o.getVertices());
		//KDTree kdt = new KDTree(4);
		
		//System.out.println("Done with kd-tree.");
		
		/*previous_size = vd.getVerticesSize();
		vd.addObject(	kdt.get_drawingStructure(), 
						kdt.get_drawingStructureUV(), 
						kdt.get_drawingStructure().length/3, 
						false);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		//s.translate(-20.0f, -20.0f, -20.0f);
		//s.scale(40);
		objects.add(s);*/
		
		//CUBE START
		o = new ObjectLoader("quad");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.translate(0.0f, 0.0f, -4.0f);
		objects.add(s);
		
		o = new ObjectLoader("quad");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.translate(0.0f, 0.0f, -2.0f);
		objects.add(s);
		
		o = new ObjectLoader("quad");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.rotateY((float)Math.PI/2);
		s.translate(1.0f, 0.0f, -3.0f);
		objects.add(s);
		
		o = new ObjectLoader("quad");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.rotateY((float)Math.PI/2);
		s.translate(-1.0f, 0.0f, -3.0f);
		objects.add(s);
		//CUBE END
		
		//SECLECTION START
		o = new ObjectLoader("quad");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.color = new float[]{0.9f, 0.3f, 0.3f};
		s.scale(0.4f);
		s.rotateY((float)Math.PI/2);
		s.translate(-1.01f, 0.0f, -3.0f);
		//objects.add(s);
		
		o = new ObjectLoader("quad");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.color = new float[]{0.9f, 0.3f, 0.3f};
		s.scale(0.4f);
		s.translate(0.0f, 0.0f, -1.99f);
		//objects.add(s);
		
		o = new ObjectLoader("cube");
		previous_size = vd.getVerticesSize();
		vd.addObject(o.getVertices(), o.getTextCoords(), null, o.getSize(), true);
		s = new SceneObject(GL11.GL_QUADS, SceneObject.WIREFRAME, previous_size, vd.getVerticesSize());
		s.shader = ShaderCollection.BASIC;
		s.color = new float[]{0.9f, 0.3f, 0.3f};
		s.scale(0.4f);
		s.translate(0.0f, 0.0f, -3.00f);
		//objects.add(s);
		//SELECTION END
		
		//this.loadFilterInput();
		//loadFilterInput("data/test.png", new int[]{100, 100}, false, 0.05f, 0.4f);
		
		//required for exclusion from ray-casting...
		//vd.reorderObjects(); //TODO: reordering not yet implemented
		//apply transformations (to the raw vertex data)
		for (int i=0;i<objects.size(); i++) {
			objects.get(i).apply_object_transformation(vd);
		}
		
		was_Load = true;
		
	}
		
	public VertexData get_vertex_data() {
		return vd;
	}
	
	public CameraFPS get_current_camera() {
		return c;
	}
	
	public ArrayList<SceneObject> get_scene_object() {
		return objects;
	}
	
	public void activateLoading(FilterParameters fp, boolean filter) {
		if (!active_loading) {
			f_name = fp.filename;
			f_size = fp.size;
			f_filter = filter;
			f_sig_s = fp.sigma_s;
			f_sig_r = fp.sigma_r;
			active_loading = true;
		}
	}
	
	public boolean isLoading() {
		return active_loading;
	}
	
	public void run() {
		
		while(!isInterrupted()) {
		
			if (!was_Load) {
			
				loadScene();
				
				if (Entrance.DEBUG) System.out.println("Scene: Done loading.");
				
			} else if (active_loading) {
				
				System.out.println("Started a loading process...");
				
				loadFilterInput(f_name, f_size, f_filter, f_sig_s, f_sig_r);
				
				System.out.println("Done with loading.");
				
				active_loading = false;
				
			}
			
		}
	}
	
}
