package world;

/**
 *
 * Scene Objects
 * 
 * @author DarkMatt
 *
 */

public class SceneObject {
	
	//drawing modes
	public static final int WIREFRAME = 0;
	public static final int SHADED = 1;
	public static final int PARTICLE_R = 2;
	
	public boolean visible; //can not be the same as the drawing mode
	
	public float[] position;
	public float[] rotation;
	
	public int shader;
	public float[] color;
	
	public int drawingmode, startIndex, endIndex;
	private int type;
	
	public SceneObject(int type, int drawingmode, int startIndex, int endIndex) {
		this.type = type;
		this.drawingmode = drawingmode;
		this.startIndex = startIndex;
		this.endIndex = endIndex;
		position = new float[] {0.0f, 0.0f, 0.0f};
		rotation = new float[] {1.0f, 0.0f, 0.0f,
								0.0f, 1.0f, 0.0f,
								0.0f, 0.0f, 1.0f};
		color = new float[]{0.2f, 0.2f, 0.2f};
		visible = true;
	}
	
	public void apply_object_transformation(VertexData vd) {
		
		float[] currv = new float[3];
		
		for (int i=startIndex; i<endIndex; i++) {
			currv[0] = vd.getVertices()[3*i];
			currv[1] = vd.getVertices()[3*i+1];
			currv[2] = vd.getVertices()[3*i+2];
			
			vd.getVertices()[3*i] 
					= currv[0]*rotation[0] + currv[1]*rotation[1] + currv[2]*rotation[2] + position[0];
			vd.getVertices()[3*i+1] 
					= currv[0]*rotation[3] + currv[1]*rotation[4] + currv[2]*rotation[5] + position[1];
			vd.getVertices()[3*i+2] 
					= currv[0]*rotation[6] + currv[1]*rotation[7] + currv[2]*rotation[8] + position[2];
		}
		
		position = new float[] {0.0f, 0.0f, 0.0f};
		rotation = new float[] {1.0f, 0.0f, 0.0f,
								0.0f, 1.0f, 0.0f,
								0.0f, 0.0f, 1.0f};
	}
	
	public void translate(float x, float y, float z) {
		position[0] += x;
		position[1] += y;
		position[2] += z;
	}
	
	public void scale(float s) {
		rotation[0] *= s;
		rotation[4] *= s;
		rotation[8] *= s;
	}
	
	//1		0		0
	//0		cos		-sin
	//0		sin		cos
	public void rotateX(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
		
		float[] r = new float[9];
		r[0]=rotation[0]; 					r[1]=rotation[1]; 					r[2]=rotation[2];
		r[3]=c*rotation[3]-s*rotation[6];	r[4]=c*rotation[4]-s*rotation[7]; 	r[5]=c*rotation[5]-s*rotation[8];
		r[6]=s*rotation[3]+c*rotation[6];	r[7]=s*rotation[4]+c*rotation[7]; 	r[8]=s*rotation[5]+c*rotation[8];
		
		for (int i=0; i<9; i++) rotation[i] = r[i];
	}
	
	//cos	0		sin
	//0		1		0
	//-sin	0		cos
	public void rotateY(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
			
		float[] r = new float[9];
		r[0]=c*rotation[0]+s*rotation[6]; 	r[1]=c*rotation[1]+s*rotation[7]; 	r[2]=c*rotation[2]+s*rotation[8];
		r[3]=rotation[3];					r[4]=rotation[4]; 					r[5]=rotation[5];
		r[6]=-s*rotation[0]+c*rotation[6];	r[7]=-s*rotation[1]+c*rotation[7]; 	r[8]=-s*rotation[2]+c*rotation[8];
			
		for (int i=0; i<9; i++) rotation[i] = r[i];
	}
	
	//cos	-sin	0
	//sin	cos		0
	//0		0		1
	public void rotateZ(float angle) {
		float c = (float) Math.cos(angle);
		float s = (float) Math.sin(angle);
				
		float[] r = new float[9];
		r[0]=c*rotation[0]-s*rotation[3]; 	r[1]=c*rotation[1]-s*rotation[4]; 	r[2]=c*rotation[2]-s*rotation[5];
		r[3]=s*rotation[0]+c*rotation[3];	r[4]=s*rotation[1]+c*rotation[4]; 	r[5]=s*rotation[2]+c*rotation[5];
		r[6]=rotation[6];					r[7]=rotation[7]; 					r[8]=rotation[8];
				
		for (int i=0; i<9; i++) rotation[i] = r[i];
	}
	
	public int getType() {
		return type;
	}
	
	public void print_translation() {
		System.out.println(position[0] + "/" + position[1] + "/" + position[2]);
	}
	
}
